from pims_api_client.configurator import Configurator
from pims_api_client.client import BaseApiClient

class BaseExecutor:
    client: BaseApiClient

    def __init__(self, configurator: Configurator):
        self.configurator = configurator
        self.client = BaseApiClient(configurator)


from typing import Optional
from pydantic import BaseModel


class AttributeFilter(BaseModel):
    page: int
    per_page: int
    name: Optional[str] = None
    type: Optional[str] = None
    group: Optional[str] = None


class AttributeListItem(BaseModel):
    id: str
    group: dict
    value: dict
    name: str
    type: str
    type_translation: str
    code: str
    is_required: bool
    is_editable: bool
    is_filterable: bool
    is_visible: bool
    position: int


class AttributeGroupItem(BaseModel):
    id: str
    name: str
    code: str
    position: int


default_attribute_filter = AttributeFilter(page=1, per_page=20)


from .configurator import Configurator
from .methods.attributes import AttributeMethodsExecutor
from .methods.products import ProductsMethodsExecutor


class Api:
    def __init__(self, configurator: Configurator):
        self.configurator = configurator

    @property
    def attributes(self) -> AttributeMethodsExecutor:
        return AttributeMethodsExecutor(self.configurator)

    @property
    def products(self) -> ProductsMethodsExecutor:
        return ProductsMethodsExecutor(self.configurator)


from pydantic import BaseModel
from pydantic.generics import GenericModel
from typing import Generic, TypeVar, List

T = TypeVar('T')


class PerPageRequest(BaseModel):
    per_page: int = 25
    page: int = 1


class PagedListResponse(GenericModel, Generic[T]):
    count: int
    per_page: int
    page: int
    pages: int
    items: List[T]



from typing import Optional

class Configurator:
    base_url: Optional[str] = None
    bearer_token: Optional[str] = None
    options: dict = {}

    @classmethod
    def build(cls, bearer_token, base_url):
        instance = cls()
        instance.base_url = base_url
        instance.bearer_token = bearer_token
        instance.validate()
        return instance

    def validate(self) -> bool:
        assert self.base_url is not None, 'base_url не указан'
        assert self.bearer_token is not None, 'bearer token не указан'
        return True



from pims_api_client.schemas import AttributeFilter, AttributeListItem
from pims_api_client.schemas import PagedListResponse
from typing import Optional
from .base_executor import BaseExecutor
from ..schemas.attributes import default_attribute_filter


class AttributeMethodsExecutor(BaseExecutor):

    async def fetch_attributes(self, params: AttributeFilter = default_attribute_filter):

        response = await self.client.get('pim/api/v1/attribute', params=params.dict())
        return PagedListResponse[AttributeListItem].parse_obj(response)

    async def fetch_attribute(self, uid: str):
        response = await self.client.get(f'pim/api/v1/attribute/{uid}')
        return AttributeListItem.parse_obj(response)



from .configurator import Configurator
from .exceptions import raise_the_desired_error

from typing import Any

import httpx
from humps.main import decamelize, camelize
from urllib.parse import urljoin


class BaseApiClient:

    def __init__(self, configurator: Configurator):
        self.configurator = configurator

    @property
    def _headers(self):
        return {
            "Authorization": 'Bearer {}'.format(self.configurator.bearer_token),
            "Content-Type": 'application/json',
            "Accept": 'application/json'
        }

    async def post(self, path:str, data={}, timeout=20, **kwargs):
        async with httpx.AsyncClient() as client:
            response = await client.post(
                urljoin(self.configurator.base_url, path),
                headers=self._headers,
                data=self._prepare_data(data),
                timeout=timeout
            )

        return self._process_response(response)

    async def get(self, path:str, params={}, timeout=20, **kwargs):
        async with httpx.AsyncClient() as client:
            response = await client.get(
                urljoin(self.configurator.base_url, path),
                headers=self._headers,
                params=self._prepare_data(params),
                timeout=timeout
            )

        return self._process_response(response)

    def _prepare_data(self, data: dict) -> dict:
        cache = {k: v for k, v in data.items() if v}
        return camelize(cache)

    def _process_response(self, response: Any) -> dict:
        try:
            response.raise_for_status()
        except httpx.HTTPStatusError as exc:
            raise raise_the_desired_error(exc)

        return decamelize(response.json())


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pims_api_client', 'pims_api_client.methods', 'pims_api_client.schemas']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.22.0,<0.23.0',
 'pydantic>=1.9,<2.0',
 'pyhumps>=3.5.3,<4.0.0',
 'pytest-asyncio>=0.18.2,<0.19.0']

setup_kwargs = {
    'name': 'pims-api-client',
    'version': '0.1.1',
    'description': 'INLB PIMS API Client',
    'long_description': None,
    'author': 'Никита Злаин',
    'author_email': 'nz@inlb.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.inlb.ru/inlb/python/pims-api-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from .base_executor import BaseExecutor
from pims_api_client.schemas.products import ProductFilterItem

class ProductsMethodsExecutor(BaseExecutor):

    async def fetch_filters(self):
        response = await self.client.get('pim/api/v1/product/filter')
        return ProductFilterItem.parse_obj(response)





class HttpApiBaseException(Exception):
    def __init__(self, status_code):
        self.status_code = status_code
        super().__init__()

    def __str__(self):
        return str(self.status_code)



class Api4xxException(HttpApiBaseException):
    pass


class Api5xxException(HttpApiBaseException):
    pass


class Api404Exception(Api4xxException):
    def __init__(self):
        super().__init__(status_code=404)


class Api401Exception(Api4xxException):
    def __init__(self):
        super().__init__(status_code=401)


class Api500Exception(Api5xxException):
    def __init__(self):
        super().__init__(status_code=500)



def raise_the_desired_error(exception):
    if exception.response.status_code == 404:
        raise Api404Exception()
    elif exception.response.status_code == 500:
        raise Api500Exception()
    elif exception.response.status_code == 401:
        raise Api401Exception()

    raise HttpApiBaseException(status_code=exception.response.status_code)
